//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//
// Program        : Sorting Tester
//
// Author         : Richard E. Pattis
//                  Computer Science Department
//                  Carnegie Mellon University
//                  5000 Forbes Avenue
//                  Pittsburgh, PA 15213-3891
//                  e-mail: pattis@cs.cmu.edu
//
// Maintainer     : Author
//
//
// Description:
//
//   Test various sorting methods for correctness and speed.
//
// Known Bugs     : For correctness, must check for permutation
//
// Future Plans   : None
//
// Program History:
//   11/05/01: R. Pattis - Operational in Java for 15-111
//
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////


import edu.cmu.cs.pattis.cs151xx.Prompt;
import edu.cmu.cs.pattis.cs151xx.Timer;

import java.util.Arrays;
import java.util.Comparator;


public class Application {
	
	
	public static class IntegerIncreasing implements Comparator {
  public int compare(Object o1, Object o2)
  {return ((Integer)o2).compareTo((Integer)o1);}
}

 public static void print (Object[] a)
 {
   for (int i=0; i<a.length; i++)
     System.out.print(a[i]+" ");
   System.out.println();
 }
 

 
public static void makeNewArray()
 {
   int used = Prompt.forInt("  Enter size of array to create");
   original = new Object[used];
   
   char selection = Prompt.forChar("  z)ero, a)ascending, d)escending, r)andom, h)and picked","zadrh",'r');
   if (selection == 'z')
     for (int i=0; i<used; i++)
       original[i] = new Integer(0);
   else if (selection == 'a')
     for (int i=0; i<used; i++)
       original[i] = new Integer(i);
   else if (selection == 'a')
     for (int i=used-1; i>=0; i--)
       original[i] = new Integer(i);
   else if (selection == 'r') {
     for (int i=0; i<used; i++)
       original[i] = new Integer(i);
     for (int i=0; i<used-1; i++)
       Sort.swap(original,i,i+(int)((used-i)*Math.random()));
   }
       
   else if (selection == 'h')
      for (int i=0; i<used; i++)
       original[i] = new Integer(Prompt.forInt("["+i+"]= "));
 
   echo   = Prompt.forBoolean("  Print before/after values",false);
   verify = Prompt.forBoolean("  Verify Array is Sorted   ",true);
   
 }


 
 public static void reportIfNotSorted(Object[] o, Object[] s)
 {
   System.out.println("\nVerifying permuation and order");
   Object[] systemSorted = (Object[])o.clone();
   Arrays.sort(systemSorted,new IntegerIncreasing());
   for (int i=0; i<s.length; i++)
     if (!systemSorted[i].equals(s[i])) {
       System.out.println("  Sort Failed: at index! "+i+"\n\n");
       return;
     }
          
   System.out.println("Permuation and order verified");
 }
 
 
 
 public static void timeBubbleSort()
 {
	  System.out.println();
	  
	  if (echo) {
	    System.out.println("Before: ");
	    print(original);
	  }
	  
	  Object[] sorted = null;
	  
	  double totalTime = 0.;
	  int    repeat    = Prompt.forInt("\n\n# Times to Repeat",5);
	  for (int i=1; i<=repeat; i++) {
	    sorted = (Object[])original.clone();
	    Timer cpu = new Timer();
	    cpu.start();
	    Sort.bubble(sorted,sorted.length,new IntegerIncreasing());  
	    cpu.stop();
	    System.out.println("Bubble Sort (for "+sorted.length+" values): sort time = " +
	      cpu.getElapsed() + " seconds");
	    totalTime += cpu.getElapsed();
	  }
	  
	  
	  System.out.println("\nAverage time = " + totalTime/repeat + " seconds");

	  if (echo) { 
	    System.out.println("\n\nAfter: ");
	    print(sorted);
	  }
	    
	  if (verify)
	    reportIfNotSorted(original, sorted);
  }



 public static void timeSelectionSort()
 {
	  System.out.println();
	  
	  if (echo) {
	    System.out.println("Before: ");
	    print(original);
	  }
	  
	  Object[] sorted = null;
	  
	  double totalTime = 0.;
	  int    repeat    = Prompt.forInt("\n\n# Times to Repeat",5);
	  for (int i=1; i<=repeat; i++) {
	    sorted = (Object[])original.clone();
	    Timer cpu = new Timer();
	    cpu.start();
	    Sort.selection(sorted,sorted.length,new IntegerIncreasing());  
	    cpu.stop();
	    System.out.println("Selection Sort (for "+sorted.length+" values): sort time = " +
	      cpu.getElapsed() + " seconds");
	    totalTime += cpu.getElapsed();
	  }
	  
	  
	  System.out.println("\nAverage time = " + totalTime/repeat + " seconds");

	  if (echo) { 
	    System.out.println("\n\nAfter: ");
	    print(sorted);
	  }
	    
	  if (verify)
	    reportIfNotSorted(original, sorted);
  }



 public static void timeInsertionSort()
 {
	  System.out.println();
	  
	  if (echo) {
	    System.out.println("Before: ");
	    print(original);
	  }
	  
	  Object[] sorted = null;
	  
	  double totalTime = 0.;
	  int    repeat    = Prompt.forInt("\n\n# Times to Repeat",5);
	  for (int i=1; i<=repeat; i++) {
	    sorted = (Object[])original.clone();
	    Timer cpu = new Timer();
	    cpu.start();
	    Sort.insertion(sorted,sorted.length,new IntegerIncreasing());  
	    cpu.stop();
	    System.out.println("Insertion Sort (for "+sorted.length+" values): sort time = " +
	      cpu.getElapsed() + " seconds");
	    totalTime += cpu.getElapsed();
	  }
	  
	  
	  System.out.println("\nAverage time = " + totalTime/repeat + " seconds");

	  if (echo) { 
	    System.out.println("\n\nAfter: ");
	    print(sorted);
	  }
	    
	  if (verify)
	    reportIfNotSorted(original, sorted);
  }



 public static void timeMergeSort()
 {
	  System.out.println();
	  
	  if (echo) {
	    System.out.println("Before: ");
	    print(original);
	  }
	  
	  Object[] sorted = null;
	  
	  double totalTime = 0.;
	  int    repeat    = Prompt.forInt("\n\n# Times to Repeat",5);
	  for (int i=1; i<=repeat; i++) {
	    sorted = (Object[])original.clone();
	    Timer cpu = new Timer();
	    cpu.start();
	    Sort.merge(sorted,sorted.length,new IntegerIncreasing());  
	    cpu.stop();
	    System.out.println("Merge Sort (for "+sorted.length+" values): sort time = " +
	      cpu.getElapsed() + " seconds");
	    totalTime += cpu.getElapsed();
	  }
	  
	  
	  System.out.println("\nAverage time = " + totalTime/repeat + " seconds");

	  if (echo) { 
	    System.out.println("\n\nAfter: ");
	    print(sorted);
	  }
	    
	  if (verify)
	    reportIfNotSorted(original, sorted);
  }



 public static void timeQuickSort()
 {
	  System.out.println();
	  
	  if (echo) {
	    System.out.println("Before: ");
	    print(original);
	  }
	  
	  Object[] sorted = null;
	  
	  double totalTime = 0.;
	  int    repeat    = Prompt.forInt("\n\n# Times to Repeat",5);
	  for (int i=1; i<=repeat; i++) {
	    sorted = (Object[])original.clone();
	    Timer cpu = new Timer();
	    cpu.start();
	    Sort.quick(sorted,sorted.length,new IntegerIncreasing());  
	    cpu.stop();
	    System.out.println("Quick Sort (for "+sorted.length+" values): sort time = " +
	      cpu.getElapsed() + " seconds");
	    totalTime += cpu.getElapsed();
	  }
	  
	  
	  System.out.println("\nAverage time = " + totalTime/repeat + " seconds");

	  if (echo) { 
	    System.out.println("\n\nAfter: ");
	    print(sorted);
	  }
	    
	  if (verify)
	    reportIfNotSorted(original, sorted);
  }



 public static void timeHeapSort()
 {
	  System.out.println();
	  
	  if (echo) {
	    System.out.println("Before: ");
	    print(original);
	  }
	  
	  Object[] sorted = null;
	  
	  double totalTime = 0.;
	  int    repeat    = Prompt.forInt("\n\n# Times to Repeat",5);
	  for (int i=1; i<=repeat; i++) {
	    sorted = (Object[])original.clone();
	    Timer cpu = new Timer();
	    cpu.start();
	    Sort.heap(sorted,sorted.length,new IntegerIncreasing());  
	    cpu.stop();
	    System.out.println("Heap Sort (for "+sorted.length+" values): sort time = " +
	      cpu.getElapsed() + " seconds");
	    totalTime += cpu.getElapsed();
	  }
	  
	  
	  System.out.println("\nAverage time = " + totalTime/repeat + " seconds");

	  if (echo) { 
	    System.out.println("\n\nAfter: ");
	    print(sorted);
	  }
	    
	  if (verify)
	    reportIfNotSorted(original, sorted);
 }

 


 public static void timeSystemSort()
 {
	  System.out.println();
	  
	  if (echo) {
	    System.out.println("Before: ");
	    print(original);
	  }
	  
	  Object[] sorted = null;
	  
	  double totalTime = 0.;
	  int    repeat    = Prompt.forInt("\n\n# Times to Repeat",5);
	  for (int i=1; i<=repeat; i++) {
	    sorted = (Object[])original.clone();
	    Timer cpu = new Timer();
	    cpu.start();
	    Arrays.sort(sorted,new IntegerIncreasing());  
	    cpu.stop();
	    System.out.println("System Sort (for "+sorted.length+" values): sort time = " +
	      cpu.getElapsed() + " seconds");
	    totalTime += cpu.getElapsed();
	  }
	  
	  
	  System.out.println("\nAverage time = " + totalTime/repeat + " seconds");

	  if (echo) { 
	    System.out.println("\n\nAfter: ");
	    print(sorted);
	  }
	    
	  if (verify)
	    reportIfNotSorted(original, sorted);
 }

 


  //Present menu and get user's choice
  
  private static char menuPrompt ()
  {
    System.out.println("\n\nO(n^2) Sorts      O(nLogn) Sorts    General");
    System.out.println("  b - bubble        m - merge         n - new array");
    System.out.println("  s - selection     q - quick         Q - quit");
    System.out.println("  i - insertion     h - heap");
    System.out.println("                    A - Arrays.sort");
    
    return Prompt.forChar("\nEnter Command","bsimqhAnQ");
  }
  
  
  
  ////////////////
  //Driver Program
  ////////////////
  
	public static void main(String[] args)
	{
	  try {
	   System.out.println("Enter the following information to start.");
	   System.out.println("If unfamiliar with this program, choose a");
	   System.out.println("small array size (1000) and experiment.");
     makeNewArray();	  
     for (;;) {
       char selection = menuPrompt();
       if (selection == 'b')
         timeBubbleSort();
       else if (selection == 's')
         timeSelectionSort();
       else if (selection == 'i')
         timeInsertionSort();
       else if (selection == 'm')
         timeMergeSort();
       else if (selection == 'q')
         timeQuickSort();
       else if (selection == 'h')
         timeHeapSort();
       else if (selection == 'A')
         timeSystemSort();
       else if (selection == 'n')
         makeNewArray();
       else if (selection == 'Q')
         break;
       else
         System.out.println("Unknown selection");
     }
         
		}catch (Exception e) {
	    e.printStackTrace();
	    System.out.println("Application class terminating");
	    System.exit(0);  
	  }
  }
  
  private static boolean  echo;
  private static boolean  verify;
  private static Object[] original;
}
